/***************************************************************************
                          message.cpp  -  description
                             -------------------
    begin                : Wed Nov 28 2001
    copyright            : (C) 2001, 2002 by Thomas Friedrichsmeier
    email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "defines.h"

#include "message.h"

#include <qcanvas.h>
#include <qstring.h>
#include <qrect.h>
#include <qobject.h>

#include <klocale.h>

#include "taxipilot.h"
#include "taxipilotapp.h"
#include "cdp.h"

Message::Message() {
	phase = 4;			// which is the phase, in which the two text-sprites are not existent. See destructor
}

Message::~Message(){
	if (phase != 4) {
		delete ack_sprite;
		delete message_sprite;
	}
}

void Message::display_message (QString & message, QString & ack_key) {
	phase = 0;

	ack_sprite = new QCanvasSprite (cdp->createTextPixmapArray( i18n ( "Press '" ) + ack_key + "'...", SPACE_TEXT_FONT,
							Qt::SPACE_TEXT_COLOR, Qt::AlignHCenter | Qt::AlignTop,
							(view->width () * 0.3), (view->height () * 0.04), true, true, true), canvas);
	ack_sprite->setZ(MESSAGE_Z);

	message_sprite = new QCanvasSprite (cdp->createTextPixmapArray(message, MESSAGE_FONT,
							Qt::MESSAGE_COLOR, Qt::AlignHCenter | Qt::AlignBottom,
							(view->width () * 0.8), (view->height () * 0.4), true, true, true), canvas);
	message_sprite->setZ(MESSAGE_Z);

// Optimize Canvas-Chunksize
	QRect bound = message_sprite->boundingRect ();
	old_chunk_size = canvas->chunkSize ();
	canvas->retune ( 16 + ((int) ((bound.width () + 4* bound.height()) / 5) / 16) * 16 );

// Move the texts to bottom center of view
	message_sprite->move((view->width() / 2) + view->contentsX (), (view->height() + (bound.height() / 2)) + view->contentsY ());
	text_dest_mid = (view->height() / 2) + view->contentsY ();
	text_dest_top = view->contentsY () - (bound.height() / 2);
	message_sprite->show();

	bound = ack_sprite->boundingRect();
	ack_sprite->move((view->width() / 2) + view->contentsX (), (view->height() + (bound.height() / 2)) + view->contentsY ());

	message_sprite->setYVelocity (- TEXT_SPEED/cdp->fps () );
	ack_sprite->setYVelocity (- 2 * TEXT_SPEED/cdp->fps () );

	pause_tm->startTimer (this, 1000/cdp->fps ());
}

void Message::timerTick(int id) {

	if (message_sprite->y() > text_dest_mid) {
		message_sprite->advance (1);
	} else {
		if (!phase) {
			phase = 1;
			ack_sprite->show ();
		} else if (phase == 1) {
			ack_sprite->advance (1);
			if (ack_sprite->y() < (text_dest_mid + 20)) {
				phase = 2;
			}
		} else if (phase == 3) {
			message_sprite->advance (1);
			ack_sprite->advance (1);
			if ((ack_sprite->y() < -20) && (message_sprite->y() < text_dest_top)) {
				ack_sprite->hide();
				message_sprite->hide();
				delete ack_sprite;
				delete message_sprite;
				phase = 4;
				pause_tm->killTimers (this);
				tp->do_event_after_message ();
				canvas->retune (old_chunk_size);
			}
		}
	}
	if (phase != 2) {
		canvas->update ();
	}
}

/** Message was acknowledged */
void Message::acknowledged(){
	if (phase < 3) {
		phase = 3;
		ack_sprite->show ();	// might not have been shown yet
	}
}

/** Kills any message that might still be acitve (called when game gets aborted) */
void Message::kill (){
	if (phase != 4) {
		delete ack_sprite;
		delete message_sprite;
		pause_tm->killTimers (this);
		canvas->retune (old_chunk_size);
		phase = 4;
	}
}
